@@ -131,7 +131,7 @@ tracker_extract_module_manager_init (void)
 		files = g_list_insert_sorted (files, (gpointer) name, (GCompareFunc) g_strcmp0);
 	}
 
-	g_message ("Loading extractor rules...");
+	g_message ("Loading extractor rules... (%s)", extractors_dir);
 
 	for (l = files; l; l = l->next) {
 		GKeyFile *key_file;
@@ -155,6 +155,8 @@ tracker_extract_module_manager_init (void)
 			continue;
 		}
 
+		g_debug ("  Loaded rule '%s'", name);
+
 		g_key_file_free (key_file);
 		g_free (path);
 	}
@@ -398,6 +400,8 @@ initialize_first_module (TrackerMimetypeInfo *info)
  *          (allow-none): A #TrackerMimetypeInfo holding the information
  *          about the different modules handling @mimetype, or %NULL if
  *          no modules handle @mimetype.
+ *
+ * Since: 0.12
  **/
 TrackerMimetypeInfo *
 tracker_extract_module_manager_get_mimetype_handlers (const gchar *mimetype)
@@ -437,6 +441,8 @@ tracker_extract_module_manager_get_mimetype_handlers (const gchar *mimetype)
  * module thread awareness description.
  *
  * Returns: The %GModule currently pointed to by @info.
+ *
+ * Since: 0.12
  **/
 GModule *
 tracker_mimetype_info_get_module (TrackerMimetypeInfo          *info,
@@ -467,6 +473,8 @@ tracker_mimetype_info_get_module (TrackerMimetypeInfo          *info,
  * Iterates to the next module handling the mimetype.
  *
  * Returns: %TRUE if there is a next module.
+ *
+ * Since: 0.12
  **/
 gboolean
 tracker_mimetype_info_iter_next (TrackerMimetypeInfo *info)
