@@ -157,15 +157,15 @@ extract_oasis_content (const gchar          *uri,
 }
 
 G_MODULE_EXPORT gboolean
-tracker_extract_get_metadata (const gchar          *uri,
-                              const gchar          *mime_used,
-                              TrackerSparqlBuilder *preupdate,
-                              TrackerSparqlBuilder *metadata,
-                              GString              *where)
+tracker_extract_get_metadata (TrackerExtractInfo *extract_info)
 {
+	TrackerSparqlBuilder *metadata;
 	TrackerConfig *config;
 	ODTMetadataParseInfo info;
 	ODTFileType file_type;
+	GFile *file;
+	gchar *uri;
+	const gchar *mime_used;
 	GMarkupParseContext *context;
 	GMarkupParser parser = {
 		xml_start_element_handler_metadata,
@@ -179,6 +179,12 @@ tracker_extract_get_metadata (const gchar          *uri,
 		maximum_size_error_quark = g_quark_from_static_string ("maximum_size_error");
 	}
 
+	metadata = tracker_extract_info_get_metadata_builder (extract_info);
+	mime_used = tracker_extract_info_get_mimetype (extract_info);
+
+	file = tracker_extract_info_get_file (extract_info);
+	uri = g_file_get_uri (file);
+
 	/* Setup conf */
 	config = tracker_main_get_config ();
 
@@ -220,6 +226,8 @@ tracker_extract_get_metadata (const gchar          *uri,
 	                       file_type,
 	                       metadata);
 
+	g_free (uri);
+
 	return TRUE;
 }
 
@@ -292,6 +300,11 @@ xml_text_handler_metadata (GMarkupParseContext  *context,
 	data = user_data;
 	metadata = data->metadata;
 
+	if (text_len == 0) {
+		/* ignore empty values */
+		return;
+	}
+
 	switch (data->current) {
 	case ODT_TAG_TYPE_TITLE:
 		if (data->title_already_set) {
@@ -346,9 +359,11 @@ xml_text_handler_metadata (GMarkupParseContext  *context,
 
 	case ODT_TAG_TYPE_CREATED:
 		date = tracker_date_guess (text);
-		tracker_sparql_builder_predicate (metadata, "nie:contentCreated");
-		tracker_sparql_builder_object_unvalidated (metadata, date);
-		g_free (date);
+		if (date) {
+			tracker_sparql_builder_predicate (metadata, "nie:contentCreated");
+			tracker_sparql_builder_object_unvalidated (metadata, date);
+			g_free (date);
+		}
 		break;
 
 	case ODT_TAG_TYPE_GENERATOR:
