@@ -62,6 +62,9 @@ get_file_content (GFile *file,
 	path = g_file_get_path (file);
 
 	fd = g_open (path, O_RDONLY | O_NOATIME, 0);
+	if (fd == -1 && errno == EPERM) {
+		fd = g_open (path, O_RDONLY, 0);
+	}
 
 	if (fd == -1) {
 		g_message ("Could not open file '%s': %s",
