@@ -58,6 +58,9 @@ static gboolean document_files;
 static gboolean emails;
 static gboolean contacts;
 static gboolean feeds;
+static gboolean software;
+static gboolean software_categories;
+static gboolean bookmarks;
 static gboolean print_version;
 
 static GOptionEntry entries[] = {
@@ -74,7 +77,7 @@ static GOptionEntry entries[] = {
 	  NULL
 	},
 	{ "detailed", 'd', 0, G_OPTION_ARG_NONE, &detailed,
-	  N_("Show URNs for results (doesn't apply to --music-albums, --music-artists, --feeds)"),
+	  N_("Show URNs for results (doesn't apply to --music-albums, --music-artists, --feeds, --software, --software-categories)"),
 	  NULL
 	},
 	{ "all", 'a', 0, G_OPTION_ARG_NONE, &all,
@@ -98,7 +101,7 @@ static GOptionEntry entries[] = {
 	  NULL
 	},
 	{ "music-artists", 0, 0, G_OPTION_ARG_NONE, &music_artists,
-	  N_("Search for music artists (--all has no effect on this) "),
+	  N_("Search for music artists (--all has no effect on this)"),
 	  NULL
 	},
 	{ "images", 'i', 0, G_OPTION_ARG_NONE, &image_files,
@@ -121,8 +124,20 @@ static GOptionEntry entries[] = {
 	  N_("Search for contacts"),
 	  NULL
 	},
+	{ "software", 0, 0, G_OPTION_ARG_NONE, &software,
+	  N_("Search for software (--all has no effect on this)"),
+	  NULL
+	},
+	{ "software-categories", 0, 0, G_OPTION_ARG_NONE, &software_categories,
+	  N_("Search for software categories (--all has no effect on this)"),
+	  NULL
+	},
 	{ "feeds", 0, 0, G_OPTION_ARG_NONE, &feeds,
-	  N_("Search for feeds (--all has no effect on this) "),
+	  N_("Search for feeds (--all has no effect on this)"),
+	  NULL
+	},
+	{ "bookmarks", 'b', 0, G_OPTION_ARG_NONE, &bookmarks,
+	  N_("Search for bookmarks (--all has no effect on this)"),
 	  NULL
 	},
 	{ "version", 'V', 0, G_OPTION_ARG_NONE, &print_version,
@@ -834,6 +849,88 @@ get_music_albums (TrackerSparqlConnection *connection,
 	return TRUE;
 }
 
+static gboolean
+get_bookmarks (TrackerSparqlConnection *connection,
+               GStrv                    search_terms,
+               gint                     search_offset,
+               gint                     search_limit,
+               gboolean                 use_or_operator)
+{
+	GError *error = NULL;
+	TrackerSparqlCursor *cursor;
+	gchar *fts;
+	gchar *query;
+
+	fts = get_fts_string (search_terms, use_or_operator);
+
+	if (fts) {
+		query = g_strdup_printf ("SELECT nie:title(?urn) nie:url(?bookmark) "
+		                         "WHERE {"
+		                         "  ?urn a nfo:Bookmark ;"
+		                         "       nfo:bookmarks ?bookmark ."
+		                         "  ?urn fts:match \"%s\" . "
+		                         "} "
+		                         "ORDER BY ASC(nie:title(?urn)) "
+		                         "OFFSET %d "
+		                         "LIMIT %d",
+		                         fts,
+		                         search_offset,
+		                         search_limit);
+	} else {
+		query = g_strdup_printf ("SELECT nie:title(?urn) nie:url(?bookmark) "
+		                         "WHERE {"
+		                         "  ?urn a nfo:Bookmark ;"
+		                         "       nfo:bookmarks ?bookmark ."
+		                         "} "
+		                         "ORDER BY ASC(nie:title(?urn)) "
+		                         "OFFSET %d "
+		                         "LIMIT %d",
+		                         search_offset,
+		                         search_limit);
+	}
+
+	g_free (fts);
+
+	cursor = tracker_sparql_connection_query (connection, query, NULL, &error);
+	g_free (query);
+
+	if (error) {
+		g_printerr ("%s, %s\n",
+		            _("Could not get search results"),
+		            error->message);
+		g_error_free (error);
+
+		return FALSE;
+	}
+
+	if (!cursor) {
+		g_print ("%s\n",
+		         _("No bookmarks were found"));
+	} else {
+		gint count = 0;
+
+		g_print ("%s:\n", _("Bookmarks"));
+
+		while (tracker_sparql_cursor_next (cursor, NULL, NULL)) {
+			g_print ("  %s (%s)\n",
+			         tracker_sparql_cursor_get_string (cursor, 0, NULL),
+			         tracker_sparql_cursor_get_string (cursor, 1, NULL));
+
+			count++;
+		}
+
+		g_print ("\n");
+
+		if (count >= search_limit) {
+			show_limit_warning ();
+		}
+
+		g_object_unref (cursor);
+	}
+
+	return TRUE;
+}
+
 static gboolean
 get_feeds (TrackerSparqlConnection *connection,
            GStrv                    search_terms,
@@ -914,6 +1011,166 @@ get_feeds (TrackerSparqlConnection *connection,
 	return TRUE;
 }
 
+static gboolean
+get_software (TrackerSparqlConnection *connection,
+              GStrv                    search_terms,
+              gint                     search_offset,
+              gint                     search_limit,
+              gboolean                 use_or_operator)
+{
+	GError *error = NULL;
+	TrackerSparqlCursor *cursor;
+	gchar *fts;
+	gchar *query;
+
+	fts = get_fts_string (search_terms, use_or_operator);
+
+	if (fts) {
+		query = g_strdup_printf ("SELECT ?soft nie:title(?soft) "
+		                         "WHERE {"
+		                         "  ?soft a nfo:Software ;"
+		                         "  fts:match \"%s\" . "
+		                         "} "
+		                         "ORDER BY ASC(nie:title(?soft)) "
+		                         "OFFSET %d "
+		                         "LIMIT %d",
+		                         fts,
+		                         search_offset,
+		                         search_limit);
+	} else {
+		query = g_strdup_printf ("SELECT ?soft nie:title(?soft) "
+		                         "WHERE {"
+		                         "  ?soft a nfo:Software ."
+		                         "} "
+		                         "ORDER BY ASC(nie:title(?soft)) "
+		                         "OFFSET %d "
+		                         "LIMIT %d",
+		                         search_offset,
+		                         search_limit);
+	}
+
+	g_free (fts);
+
+	cursor = tracker_sparql_connection_query (connection, query, NULL, &error);
+	g_free (query);
+
+	if (error) {
+		g_printerr ("%s, %s\n",
+		            _("Could not get search results"),
+		            error->message);
+		g_error_free (error);
+
+		return FALSE;
+	}
+
+	if (!cursor) {
+		g_print ("%s\n",
+		         _("No software was found"));
+	} else {
+		gint count = 0;
+
+		g_print ("%s:\n", _("Software"));
+
+		while (tracker_sparql_cursor_next (cursor, NULL, NULL)) {
+			g_print ("  %s (%s)\n",
+			         tracker_sparql_cursor_get_string (cursor, 0, NULL),
+			         tracker_sparql_cursor_get_string (cursor, 1, NULL));
+
+			count++;
+		}
+
+		g_print ("\n");
+
+		if (count >= search_limit) {
+			show_limit_warning ();
+		}
+
+		g_object_unref (cursor);
+	}
+
+	return TRUE;
+}
+
+static gboolean
+get_software_categories (TrackerSparqlConnection *connection,
+                         GStrv                    search_terms,
+                         gint                     search_offset,
+                         gint                     search_limit,
+                         gboolean                 use_or_operator)
+{
+	GError *error = NULL;
+	TrackerSparqlCursor *cursor;
+	gchar *fts;
+	gchar *query;
+
+	fts = get_fts_string (search_terms, use_or_operator);
+
+	if (fts) {
+		query = g_strdup_printf ("SELECT ?cat nie:title(?cat) "
+		                         "WHERE {"
+		                         "  ?cat a nfo:SoftwareCategory ;"
+		                         "  fts:match \"%s\" . "
+		                         "} "
+		                         "ORDER BY ASC(nie:title(?cat)) "
+		                         "OFFSET %d "
+		                         "LIMIT %d",
+		                         fts,
+		                         search_offset,
+		                         search_limit);
+	} else {
+		query = g_strdup_printf ("SELECT ?cat nie:title(?cat) "
+		                         "WHERE {"
+		                         "  ?cat a nfo:SoftwareCategory ."
+		                         "} "
+		                         "ORDER BY ASC(nie:title(?cat)) "
+		                         "OFFSET %d "
+		                         "LIMIT %d",
+		                         search_offset,
+		                         search_limit);
+	}
+
+	g_free (fts);
+
+	cursor = tracker_sparql_connection_query (connection, query, NULL, &error);
+	g_free (query);
+
+	if (error) {
+		g_printerr ("%s, %s\n",
+		            _("Could not get search results"),
+		            error->message);
+		g_error_free (error);
+
+		return FALSE;
+	}
+
+	if (!cursor) {
+		g_print ("%s\n",
+		         _("No software categories were found"));
+	} else {
+		gint count = 0;
+
+		g_print ("%s:\n", _("Software Categories"));
+
+		while (tracker_sparql_cursor_next (cursor, NULL, NULL)) {
+			g_print ("  %s (%s)\n",
+			         tracker_sparql_cursor_get_string (cursor, 0, NULL),
+			         tracker_sparql_cursor_get_string (cursor, 1, NULL));
+
+			count++;
+		}
+
+		g_print ("\n");
+
+		if (count >= search_limit) {
+			show_limit_warning ();
+		}
+
+		g_object_unref (cursor);
+	}
+
+	return TRUE;
+}
+
 static gboolean
 get_files (TrackerSparqlConnection *connection,
            GStrv                    search_terms,
@@ -1182,7 +1439,10 @@ main (int argc, char **argv)
 	}
 
 	if (!music_albums && !music_artists && !music_files &&
+	    !bookmarks &&
 	    !feeds &&
+	    !software &&
+	    !software_categories &&
 	    !image_files &&
 	    !video_files &&
 	    !document_files &&
@@ -1262,7 +1522,7 @@ main (int argc, char **argv)
 
 	g_option_context_free (context);
 
-	connection = tracker_sparql_connection_get (&error);
+	connection = tracker_sparql_connection_get (NULL, &error);
 
 	if (!connection) {
 		g_printerr ("%s: %s\n",
@@ -1375,6 +1635,33 @@ main (int argc, char **argv)
 		return success ? EXIT_SUCCESS : EXIT_FAILURE;
 	}
 
+	if (software) {
+		gboolean success;
+
+		success = get_software (connection, terms, offset, limit, or_operator);
+		g_object_unref (connection);
+
+		return success ? EXIT_SUCCESS : EXIT_FAILURE;
+	}
+
+	if (software_categories) {
+		gboolean success;
+
+		success = get_software_categories (connection, terms, offset, limit, or_operator);
+		g_object_unref (connection);
+
+		return success ? EXIT_SUCCESS : EXIT_FAILURE;
+	}
+
+	if (bookmarks) {
+		gboolean success;
+
+		success = get_bookmarks (connection, terms, offset, limit, or_operator);
+		g_object_unref (connection);
+
+		return success ? EXIT_SUCCESS : EXIT_FAILURE;
+	}
+
 	if (terms) {
 		gboolean success;
 
